#!/usr/local/bin/perl
# NCBI の Nucleotide, Nucleotide EST, Protein, データベースから
# gi numbers をダウンロードするスクリプト．

# 001_giRetriever.pl Xenoturbella_bocki

# "Xenoturbella_bocki" を例にしています．以下と同じ数の gi numbers が得られているはずです．
# http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=242395

use Bio::DB::EUtilities;
use strict;
use warnings;

#####
my $spName = "$ARGV[0]";
#####

my $db = "";
my $outDNAgi = $spName."_DNAgi.txt";
my $outESTgi = $spName."_ESTgi.txt";
my $outAAgi = $spName."_AAgi.txt";


print "\n\n\n\n\n\n";
print "############################################################\n";
print "#################### 001_giRetriever.pl ####################\n";
print "############################################################\n";

print "gi numbers for $spName\n";




##### Nucleotide DB
my $ref_uidsDNA = "";
my @uidsDNA = "";

if(-e "$outDNAgi"){
  print "$outDNAgi is already exist.\n";
} else {

  # DB を検索
  $db = "nucleotide";
  $ref_uidsDNA = &giRetriever($db);
  @uidsDNA = @$ref_uidsDNA;
  
  # プリントアウト
  print "Nucleotide ".@uidsDNA."\n";
  open(DNAOUT,">$outDNAgi");
  foreach (@uidsDNA) {
    print DNAOUT "$_\n";
  }
  close DNAOUT;
}

##### EST DB
my $ref_uidsEST = "";
my @uidsEST = "";

if(-e "$outESTgi"){
  print "$outESTgi is already exist.\n";
} else {

  # DB を検索
  $db = "nucest";
  $ref_uidsEST = &giRetriever($db);
  @uidsEST = @$ref_uidsEST;
  
  # プリントアウト
  print "NucleotideEST ".@uidsEST."\n";
  open(ESTOUT,">$outESTgi");
  foreach (@uidsEST) {
    print ESTOUT "$_\n";
  }
  close ESTOUT;
}


##### Protein DB
my $ref_uidsAA = "";
my @uidsAA = "";

if(-e "$outAAgi"){
  print "$outAAgi is already exist.\n";
} else {

  # DB を検索
  $db = "protein";
  $ref_uidsAA = &giRetriever($db);
  @uidsAA = @$ref_uidsAA;
  
  # プリントアウト
  print "Protein ".@uidsAA."\n";
  open(AAOUT,">$outAAgi");
  foreach (@uidsAA) {
    print AAOUT "$_\n";
  }
  close AAOUT;
}

print "\n\n";


exit;




#### サブルーチン

sub giRetriever{
  my($db) = @_;
  my $factory = Bio::DB::EUtilities->new(-eutil  => 'esearch',
  
                                         -db     => "$db",
                                         # protein,nucleotide,nuccore,nucgss,nucest,structure,genome,gene,journals,
                                         # Bio::Tools::EUtilities::EUtilParameters を参照．
                                         
                                         -term   => "$spName [Organism]",
  #                                       -email  => 'mymail@foo.bar',
                                         -retmax => 10000000);
  my @uids    = $factory->get_ids;
  return (\@uids);
}


# このスクリプトは以下のページを参照に作りました．
# http://search.cpan.org/~sendu/bioperl-1.5.2_102/Bio/DB/EUtilities.pm
# http://www.bioperl.org/wiki/HOWTO:EUtilities_Cookbook#How_do_I_retrieve_the_DNA_sequence_using_EntrezGene_IDs.3F
