#!/usr/bin/perl -w

# local Blast を用いてある種の EST 配列を検索・抽出するスクリプト．

# 操作       : 002_LocalBlast_ESTSeqPicker.pl Xenoturbella_bocki NucGene.nex

# インファイル: MacClade で保存した cDNA alignment
# データベース: NCBI の est_others データベース．
# 必要なソフトウェア: local Blast, mafft, pal2nal

use strict;
use warnings;

# 検索条件の設定．
##########
my $giList = $ARGV[0]."_ESTgi.txt";       # データを集めた種の gi list．
my $inFile = $ARGV[1];                    # インファイル．cDNA アライメント．

my $querySpName = "Human";                # クエリ配列に用いる種名．インファイル (cDNA アライメント) からコピーして使う．
my $eValue = 10;                          # blastall で用いる evalue． eg. my $eValue = "10"; my $eValue = "1e-62";  
my $Num_blastResSeq = 10;                 # blastall で結果として得られる配列の数を指定する．「-b」オプション．
                                          # http://www.ncbi.nlm.nih.gov/staff/tao/URLAPI/blastall/blastall_node37.html
my $SpeciesName = $ARGV[0]."_EST_";       # 種名．得られたアライメントに Name line として付け足す．

##########



print "\n\n\n\n\n\n";
print "###########################################################\n";
print "############# 002_LocalBlast_ESTSeqPicker.pl #############\n";
print "###########################################################\n";


#################### 準備 ####################



##### infile 名の拡張子を取り除く
my $inFileNoSu = "";

$inFileNoSu = $inFile;
$inFileNoSu =~ s/\..+$//g;


##### infile の改行コードを Mac から Unix にする．
system ("perl -i -pe 's/\r/\n/g' $inFile");


##### 解析条件をスクリーンアウトする．

print "\n";
print "\n";
print "############# local Blast against EST DB --- $inFile  --- #############\n";


##### ファイルの確認

if(-e $giList){
  print "$giList is found\n";
}
else {
  print "$giList is NOT found\n";
  print "LocalBlast_ESTSeqPicker.pl finished\n";
  exit;
}


print "\n";
print "############# local Blast against EST DB (Gene: $inFileNoSu, target species) #############\n";

###### infile, cDNA アライメントファイルを配列に格納する．
my @inFileCont = ( );

open(INFILE,"$inFile");
@inFileCont = <INFILE>;
close INFILE;

# クエリ種がインファイルにあるか確認する．
if(grep/$querySpName/,@inFileCont){
  print "Query $querySpName was found in $inFile\n";
} else {
  print "Query was NOT found in $inFile\n";
  exit;
}


###### infile を nexus から fasta に書き換える．
my $ref_inFileContfas = "";     # ファスタ形式の塩基配列データ．ギャップあり．

$ref_inFileContfas = &nexToFas(\@inFileCont);


#### 配列に格納されているファスタ形式の cDNA データを，塩基配列とアミノ配列としてファスタ形式で保存する．その際，ギャップを取り除く．

my $outfileAANGFas = "010_".$inFileNoSu."_AANG.fas";     # アウトファイル．ギャップなしのアミノ酸配列
my $OutNucNGFas = "020_".$inFileNoSu."_nucNG.fas";       # アウトファイル．ギャップなしの塩基配列
my $NucTransLine = "";
my $codon = "";
my $protein = "";
my @aangArray = ( );                              # アミノ酸配列
my @nucngArray = ( );                             # 塩基配列

# 配列に格納されたファスタ形式の cDNA 配列を得る
open(AANG,">$outfileAANGFas");
open(NUCNG,">$OutNucNGFas");
foreach my $cDNAline (@$ref_inFileContfas) {      

  # name line を得て，配列に格納する．
  if($cDNAline =~ /^>/){                          
    push(@aangArray,$cDNAline);
    push(@nucngArray,$cDNAline);
  } 
  
  # sequence を得て，配列に格納する．
  else {                                         # 配列の処理

    # アミノ酸配列の作成．dna2aa_Std によって翻訳．
    $NucTransLine = $cDNAline;
    for(my $i=0; $i < (length($NucTransLine) - 2) ; $i += 3) {
      $codon = substr($NucTransLine,$i,3);
      $protein .= dna2aa_Std($codon);
    }
    $protein =~ s/-//g;                #　ギャップを取り除く
    push(@aangArray,$protein);
    $protein = "";
  
    # 塩基配列の作成．
    $cDNAline =~ s/-//g;                #　ギャップを取り除く
    push(@nucngArray,$cDNAline);
  
  }
}

## プリントアウト
# アミノ酸配列
foreach (@aangArray) {
  print AANG "$_\n";
}
close AANG;

# 塩基配列
foreach (@nucngArray) {
  print NUCNG "$_\n";
}
close NUCNG;




###### 元のアミノ酸配列のファスタファイルが入った @aangArray からクエリ配列を取ってきて，ファイルとして保存する．

# AA seq ファスタファイルからクエリデータを取ってくる．
my $aastone = 0;
my $queryFileName = "030_".$inFileNoSu."_querySeq";    # アウトファイル．ファスタ形式のクエリ配列が一つだけ入っている．

print "\n";
print "query seq:\n";
open(QUERYOUT,">$queryFileName");


foreach my $aaFasline (@aangArray) {
   if($aastone == 1){                # クエリ種名の name line line をすぎたら始める．
     if($aaFasline =~ /^>/){         # クエリ種名の次の name line がきたら終わり．
       last;
     } else {                        # それ以外はクエリ種の配列なのでプリントアウトする．
       print "$aaFasline\n";
       print QUERYOUT "$aaFasline\n";      
     }
  }
  if ($aaFasline =~ />$querySpName/) {
    $aastone = 1;
    print "$aaFasline\n";
    print QUERYOUT "$aaFasline\n";
  }
}
close QUERYOUT;





######################################## メインプログラム ########################################  

################### local Blast search  #################### 

###### blastall
# AAseq をクエリとして，EST データベースを Blast seqrch．
# $giList はあらかじめ NCBI のサイトから用意しておく．
my $blastRes = "040_".$inFileNoSu."_BlastResult.txt";   # アウトファイル．local blast の結果．

print "\n";
print "### blastall started ###\n";
print "-b: $Num_blastResSeq records\n";
print "(Upper limit on number of database sequence to show alignments for)\n";
print "-e: eValue $eValue\n";
system("blastall -i $queryFileName -d NCBI/est_others/est_others -o $blastRes -p tblastn -l $giList -e $eValue -b $Num_blastResSeq");
#-b で取ってくる配列の数を指定している．

print "### blastall finished ###\n\n";



#### fastacmd
# fastacmd で該当する EST 配列を取ってくる．
# 上の blastall で得られた結果である $blastRes から，
# 塩基配列の accession number をとってきてファイルとして保存して fastacmd に使う．
# Blast で出てきたアミノ酸アライメントも保存する．

my @blastResCon = ( );        # blast の結果ファイルを格納する配列．
my $numBlastedSeqs = "";      # blast でヒットした配列の数．「^>」の数を grep して探している．

my $aaSeqLine = "";
my %BlastMachedAA = ( );       # tblastn で matche された target sequence の一部を gi num とともにハッシュにする．

my $giNum = "";
my @giNumArray = ( );          # gi を配列に格納しておく．

my $targetAAstone = "0";
my $SbjctStone = "0";
my $nameLine = "";

my $blastResNums = "050_".$inFileNoSu."_BlastResult_GiNums.txt";         # アウトファイル．tblastn の 結果の番号だけ
my $blastResNucSeqs = "070_".$inFileNoSu."_BlastResult_cDNASeqs.fas";    # アウトファイル．local Blast で拾ってきた cDNA 配列．

###

open(BLRES,"$blastRes");                # blastall の結果を開く．
@blastResCon = <BLRES>;
close BLRES;
print "### blastall results analysis ###\n";
print "blastall results, gi numbers only (picking up from outfile, $blastRes):\n";

if(grep/^>/,@blastResCon) {                      # ヒットしたレコードがあるか調べて解析を始める．
  $numBlastedSeqs = grep(/^>/,@blastResCon);
  print "## $numBlastedSeqs records hit\n\n";

  foreach my $line (@blastResCon){
  
     if($line =~ /^>/){                 # blast の結果，スクリーンアウト (fasta ではない)，name line の処理．
        $SbjctStone = 0;                # name line にくるたびに，Sbjct の最初の一行を拾うために stone をリセット．
  
       if ($targetAAstone == 0) {       # 最初の name line
  
         # gi number を得る．
         $line =~ /^>.*?\|(.*)\|.*?$/;
         $giNum = $1;                     # gi number
         $giNum =~ s/\.\d+//g;
  
         # line を name line としてとっておく
         $nameLine = $line;
  
         $targetAAstone = 1; 
       } else {                   # 二番目の name line に来たら，それぞれプリントアウト．
  
         ## 前のレコードを格納
         push(@giNumArray,$giNum);
         $BlastMachedAA{$giNum} = $aaSeqLine;
  
         ## 今回のレコードを処理．
         # gi number を得る．
         $line =~ /^>.*?\|(.*)\|.*?$/;
         $giNum = $1;                     # gi number
         $giNum =~ s/\.\d+//g;
  
         # line を name line としてとっておく
         $nameLine = $line;
  
       }
  
    } else {                          # name line 以外を処理
  
      if($line =~ /^Sbjct:/){
        if($SbjctStone == 0){           # name line のあと，最初に出てきた Sbjct line だけを拾う．
          $aaSeqLine = $line;
          chomp($aaSeqLine);
          $aaSeqLine =~ s/\d//g;
          $aaSeqLine =~ s/Sbjct://g;
          $aaSeqLine =~ s/\s//g;
          $aaSeqLine =~ s/-//g;
          $SbjctStone = 1;
        }
      }
    }
  }
  
  # 最後のレコードを格納
  push(@giNumArray,$giNum);
  $BlastMachedAA{$giNum} = $aaSeqLine;
  
  
  # プリントアウト
  open(BLRESNUMS,">$blastResNums");       # アウトファイル．blast の結果から gi num だけを抜き出して保存する．
  #open(BLRESAASEQS,">$blastResMatchedAA");   # 取ってきたアミノ酸配列を保存するファイルをあける．
  
  foreach (@giNumArray) {
    print ">$_\n$BlastMachedAA{$_}\n";
    print BLRESNUMS "$_\n";
  }
  close BLRES;
  close BLRESNUMS;
  #close BLRESAASEQS;
  
  
  # fastacmd を走らせる．gi num の書いてあるファイル $blastResNums をあけて，fastacmd に使う．
  print "\n\n### fastacmd started ###\n";
  system("fastacmd -d NCBI/est_others/est_others -i $blastResNums -o $blastResNucSeqs");
  print "### fastacmd finished ###\n\n";
  
} else {                      # 「^>」が無ければレコードがヒットしなかったので解析をストップする．
  print "## No record hit\n";
  exit;
}


################### 得られた cDNA 配列を tblastn 得られた match 配列と合うように翻訳する #################### 

my $NucTransLine_cDNA = "";   # 翻訳に用いるそれぞれの cDNA 配列
my $ComNucTransLine_cDNA = "";   # 翻訳に用いるそれぞれの cDNA 配列の相補鎖

my $codon_cDNA = "";
my $protein_cDNA = "";
my $seqLengthLef = "0";          # cDNA 配列の長さを 3 で割ったあまり．

my @inFileCont_cDNA = ( );    # EST 配列セットを保存したファイルの内容を格納する．

my $blastResAASeqs     = "075_".$inFileNoSu."_BlastResult_translatedAASeqs.fas";      # target EST 配列を翻訳して得られた AA seqs.
my $blastResNucSeqsCom = "078_".$inFileNoSu."_BlastResult_NomalComcDNASeqs.fas";    # アウトファイル．local Blast で拾ってきた cDNA 配列．


####

open(CDNARET,"$blastResNucSeqs");              # blast で得られた cDNA 配列が保存されたファイルを配列に格納．
@inFileCont_cDNA = <CDNARET>;
close CDNARET;

# fastacmd によって保存された cDNA ファスタファイルをハッシュと配列に格納．
my ($ref_fasHashSR,$ref_nameArraySR) = &fasHashReader(\@inFileCont_cDNA);

open(TRANSLATEDAA,">$blastResAASeqs");            # 翻訳したアミノ酸配列を保存．
open(COMPDNA,">$blastResNucSeqsCom");             # 適宜 revCom した cDNA を保存．
print "### cDNA translation started ###\n";

Y_LOOP: foreach my $giNumline (@giNumArray) {

  print "# $giNumline\n";

  my (@tmpArray) = grep(/$giNumline/,@$ref_nameArraySR);
  my $cDNAname = shift @tmpArray;

  
  ##### アミノ酸に翻訳する
  $NucTransLine_cDNA = $$ref_fasHashSR{$cDNAname};
  $NucTransLine_cDNA = uc $NucTransLine_cDNA;
  $seqLengthLef = length($NucTransLine_cDNA) % 3;


  ### 1st, 2nd, 3rd から翻訳する
  for(my $p=1; $p < 4; $p++){
    
    # コドンごとに翻訳
    for(my $i=$p-1; $i < (length($NucTransLine_cDNA) - 2 - $seqLengthLef) ; $i += 3) {    
      $codon_cDNA = substr($NucTransLine_cDNA,$i,3);
      $protein_cDNA .= dna2aa_Std($codon_cDNA);      
    }

    # 翻訳した配列に local Blast で hit が確認されたアミノ酸配列があればプリントアウトして次のレコード (gi number) へ．
    if ($protein_cDNA =~ /^.*\**(.*?$BlastMachedAA{$giNumline}.*?)\*/ or $protein_cDNA =~ /^.*\**(.*?$BlastMachedAA{$giNumline}.*)$/){
      print TRANSLATEDAA ">$SpeciesName$giNumline\n$1\n";
      print COMPDNA ">$giNumline\n$NucTransLine_cDNA\n";
      print ">$giNumline $p pos start\n$1\n";
      $protein_cDNA = "";
      next Y_LOOP;           # 翻訳と blast で得た AA 断片が合えば次へ
    }
  } 


  ### 相補鎖を得て 1st, 2nd, 3rd から翻訳する
  $ComNucTransLine_cDNA = reverse $NucTransLine_cDNA;
  $ComNucTransLine_cDNA =~ tr/ACGT/TGCA/;
  for(my $p=1; $p < 4; $p++) {
 
    # コドンごとに翻訳
    for(my $i=$p-1; $i < (length($ComNucTransLine_cDNA) - 2 - $seqLengthLef) ; $i += 3) {    
      $codon_cDNA = substr($ComNucTransLine_cDNA,$i,3);
      $protein_cDNA .= dna2aa_Std($codon_cDNA);      
    }

    # 翻訳した配列に local Blast で hit が確認されたアミノ酸配列があればプリントアウトして次のレコード (gi number) へ．
    if ($protein_cDNA =~ /^.*\**(.*?$BlastMachedAA{$giNumline}.*?)\*/ or $protein_cDNA =~ /^.*\**(.*?$BlastMachedAA{$giNumline}.*)$/){
      print TRANSLATEDAA ">$SpeciesName$giNumline\n$1\n";
      print COMPDNA ">$giNumline\n$ComNucTransLine_cDNA\n";
      print ">$giNumline Com$p pos start\n$1\n";
      $protein_cDNA = "";
      next Y_LOOP;           # 翻訳と blast で得た AA 断片が合えば次へ
    } 
  } 
 
 $protein_cDNA = "";
 print "\n";

}
close TRANSLATEDAA;
close COMPDNA;
print "### cDNA translation finished ###\n\n";



################### 得られた配列を元の cDNA アライメントに加えてアライメントをやり直す． #################### 

#### 得られた配列を元のファイルに加えて新しい配列として保存する．
my $blastResAASeqsAdded = "080_".$inFileNoSu."_BlastResult_AASeqsAdded.fas";      # アウトファイル．local Blast で得られたファイルに書かれている AA 配列をまとめてファイルとして保存．
my $blastResNucSeqsAdded = "090_".$inFileNoSu."_BlastResult_NucSeqsAdded.fas";    # アウトファイル．local Blast で拾ってきた塩基配列とインファイルの cDNA 配列リスト．
my @addAAseqs = ( );                                                              # blast で得られた AA seq を格納．
my @addNucseqs = ( );

# local Blast で得られた AA seq を配列に入れる．
open(ADDAASEQS,"$blastResAASeqs");
@addAAseqs = <ADDAASEQS>;
close ADDAASEQS;

# local Blast で得られ適宜 revCom した nuc seq を配列に入れる．
open(ADDNUCSEQS,"$blastResNucSeqsCom");
@addNucseqs = <ADDNUCSEQS>;
close ADDNUCSEQS;


### プリントアウト
# アミノ酸配列  mafft と pal2nal に使われる AA 配列セット．を作成．blast で得られた AA と元の AA データセットを一つのファイルに保存する．
open(TOMAFFTAA,">$blastResAASeqsAdded");
  foreach (@addAAseqs){
    print TOMAFFTAA "$_";
  }
  foreach (@aangArray){
    print TOMAFFTAA "$_\n";
  }
close TOMAFFTAA;

# 塩基配列  pal2nal に使われる cDNA 配列セット．を作成．blast で得られた cDNA と元の cDNA データセットを一つのファイルに保存する．
open(TOMAFFTNUC,">$blastResNucSeqsAdded");              
  foreach (@addNucseqs){
    print TOMAFFTNUC "$_";
  }
  foreach (@nucngArray){
    print TOMAFFTNUC "$_\n";
  }
close TOMAFFTNUC;


#### mafft と pal2nal で解析して，nexus フォーマットにする．
my $maftOut = "100_".$blastResAASeqsAdded.".maf";
my $pal2nalOut = "110_".$blastResAASeqsAdded.".p2n";

print "### mafft started ###\n";
system("mafft $blastResAASeqsAdded > $maftOut");                           #mafft
print "### mafft finished ###\n\n";

print "### pal2nal started ###\n";
system("pal2nal.pl $maftOut $blastResNucSeqsAdded -output fasta > $pal2nalOut");   #pal2nal 
print "### pal2nal finished ###\n\n";

print "### FasToNex_cDNA.pl started ###\n";
&FasToNex_cDNA($pal2nalOut);                                                 # fasta を nexus に変換

system("mv $pal2nalOut.nex $inFileNoSu\_ESTadded.nex");                       # 最終結果ファイルの名前を変える


####### 余分なアウトファイルの消去．
unlink($outfileAANGFas);         # 010
unlink($OutNucNGFas);            # 020
unlink($queryFileName);          # 030
unlink($blastRes);               # 040
unlink($blastResNums);           # 050
unlink($blastResNucSeqs);        # 070
unlink($blastResAASeqs);         # 075
unlink($blastResNucSeqsCom);     # 078
unlink($blastResAASeqsAdded);    # 080
unlink($blastResNucSeqsAdded);   # 090
unlink($maftOut);                # 100
unlink($pal2nalOut);             # 110
                                       #    塩基配列アライメント．pal2nal の結果．
#
exit;




#################################### サブルーチン ####################################

# nexToFas
#
# nexus 形式を fasta 形式に置き換える．
#

sub nexToFas    
{
my $stone = 0;     

my $name = "";     
my $seqNF = "";     
my $inFileContRF = ""; 

my @nucFasConRF = ( );

  $inFileContRF = shift @_;

  foreach my $line (@$inFileContRF) {
    if($stone == 0){
      if($line =~ /^MATRIX/){
        $stone = 1;
        next;
      } else {
        next;
      }
    }
    
    if($stone ==1){
      if($line =~ /^;/) {
        last;
      } elsif($line =~ /^\[/ || $line =~ /^\s/) {
        next;
      } else {
        $line =~ s/\[\d+\]//g;
        ($name,$seqNF) = split(/\s+/,$line);
        push (@nucFasConRF,">$name");
        push (@nucFasConRF,"$seqNF");
      }
    }
  }
  return (\@nucFasConRF);
}


# fasHashReader
#
#  Fasta 形式をハッシュに読み込む
sub fasHashReader
{
my $name = "";
my $seq = "";
my $stone = 0;
my @nameArraySR = ( );
my %fasHashSR = ( );

my ($inFileContSR) = @_;

foreach my $inFileLine (@$inFileContSR) {
  chomp($inFileLine);
  if($inFileLine =~ /^>/){  
    if($stone == 0){$name = $inFileLine;
      $stone = 1;  
    } else {
      $fasHashSR{$name} = $seq;  
      push(@nameArraySR,$name);
      $name = "$inFileLine";
      $seq = "";
    }
  } else {
    $seq .= $inFileLine; 
  }
}
$fasHashSR{$name} = $seq; 
push(@nameArraySR,$name);

# ハッシュと配列をメインプログラムに返す．
return (\%fasHashSR,\@nameArraySR);

}



#
# dna2aa_Std
#
# cDNA をアミノ酸に変換するサブルーチン

sub dna2aa_Std {
    my($codon) = @_;

#    print "$codon\n";
    
    $codon = uc $codon;
 
    my(%genetic_code) = (
    
    'TCA' => 'S',    # Serine
    'TCC' => 'S',    # Serine
    'TCG' => 'S',    # Serine
    'TCT' => 'S',    # Serine
    'TTC' => 'F',    # Phenylalanine
    'TTT' => 'F',    # Phenylalanine
    'TTA' => 'L',    # Leucine
    'TTG' => 'L',    # Leucine
    'TAC' => 'Y',    # Tyrosine
    'TAT' => 'Y',    # Tyrosine
    'TAA' => '*',    # Stop
    'TAG' => '*',    # Stop
    'TGC' => 'C',    # Cysteine
    'TGT' => 'C',    # Cysteine
    'TGA' => '*',    # Stop
    'TGG' => 'W',    # Tryptophan
    'CTA' => 'L',    # Leucine
    'CTC' => 'L',    # Leucine
    'CTG' => 'L',    # Leucine
    'CTT' => 'L',    # Leucine
    'CCA' => 'P',    # Proline
    'CCC' => 'P',    # Proline
    'CCG' => 'P',    # Proline
    'CCT' => 'P',    # Proline
    'CAC' => 'H',    # Histidine
    'CAT' => 'H',    # Histidine
    'CAA' => 'Q',    # Glutamine
    'CAG' => 'Q',    # Glutamine
    'CGA' => 'R',    # Arginine
    'CGC' => 'R',    # Arginine
    'CGG' => 'R',    # Arginine
    'CGT' => 'R',    # Arginine
    'ATA' => 'I',    # Isoleucine
    'ATC' => 'I',    # Isoleucine
    'ATT' => 'I',    # Isoleucine
    'ATG' => 'M',    # Methionine
    'ACA' => 'T',    # Threonine
    'ACC' => 'T',    # Threonine
    'ACG' => 'T',    # Threonine
    'ACT' => 'T',    # Threonine
    'AAC' => 'N',    # Asparagine
    'AAT' => 'N',    # Asparagine
    'AAA' => 'K',    # Lysine
    'AAG' => 'K',    # Lysine
    'AGC' => 'S',    # Serine
    'AGT' => 'S',    # Serine
    'AGA' => 'R',    # Arginine
    'AGG' => 'R',    # Arginine
    'GTA' => 'V',    # Valine
    'GTC' => 'V',    # Valine
    'GTG' => 'V',    # Valine
    'GTT' => 'V',    # Valine
    'GCA' => 'A',    # Alanine
    'GCC' => 'A',    # Alanine
    'GCG' => 'A',    # Alanine
    'GCT' => 'A',    # Alanine
    'GAC' => 'D',    # Aspartic Acid
    'GAT' => 'D',    # Aspartic Acid
    'GAA' => 'E',    # Glutamic Acid
    'GAG' => 'E',    # Glutamic Acid
    'GGA' => 'G',    # Glycine
    'GGC' => 'G',    # Glycine
    'GGG' => 'G',    # Glycine
    'GGT' => 'G',    # Glycine
    '---' => '-',    # gap
    'NNN' => 'N',    # gap
    );

    if(exists $genetic_code{$codon}) {
      return $genetic_code{$codon};
    }else{
#      print STDERR "Bad codon \"$codon\"!!\n";
      return "X";
#      next;
    }
}


# Fasta 形式の cDNA ファイルを Nexus 形式に変換するファイル．
# sub fasHashReader も使うので，両方ペーストすること．
sub FasToNex_cDNA
{
    my ($infile) = @_;

    open(INFILE,"$infile");
    my @inFileCont = <INFILE>;
    close INFILE;
    
    my ($ref_fasHashSR,$ref_nameArraySR, $seqLength) = &fasHashReader(\@inFileCont);
    my %efas = %$ref_fasHashSR;
    my @SpListArray = @$ref_nameArraySR;

    
    #### 配列の長さが違うときのために，最長の長さを調べて，短いものはギャップで埋める．
    # この操作は，fasta 形式の配列の長さが違う場合を考慮したもの．
    my $maxSeqLength = "";
    my @seqLength = ( );
    my @seqLengthSorted = ( );

    foreach (values %efas){
      push (@seqLength,length($_));
    }
    @seqLengthSorted = sort { $a <=> $b } @seqLength;
    #print "@seqLengthSorted\n";
    $maxSeqLength = pop(@seqLengthSorted);
    #print "max $maxSeqLength\n";
    
    
    ##### 最も長い名前の文字数を調べて，名前の後に加える空白の数を決める．
    my $maxNameLength = "";
    my @nameLength = ( );
    my @nameLengthSorted = ( );

    foreach (@SpListArray) {
      push(@nameLength,length($_));  
    }
    @nameLengthSorted = sort { $a <=> $b } @nameLength;
    #print "@nameLengthSorted\n";
    $maxNameLength = pop(@nameLengthSorted);
    #print "Max name length $maxNameLength\n";
    
    
    
    #######プリントアウト
    
    my $space = "";
    my $gap = "";
    my $outfile = $infile.".nex";

    open(OUT,">$outfile");
    
    print OUT 
"#NEXUS 

BEGIN DATA;
	DIMENSIONS  NTAX=".($#SpListArray+1)." NCHAR=$maxSeqLength;
	FORMAT DATATYPE=DNA  MISSING=? GAP=- ;

MATRIX\n";
    

    foreach my $pname (@SpListArray) {
      chomp($pname);
      $space = " " x ($maxNameLength + 5 -length($pname));
      $gap = "-" x ($maxSeqLength - length($efas{$pname}));
      my $spNameNoAR = $pname;
      $spNameNoAR =~ s/^>//g;
      print OUT "$spNameNoAR     $space $efas{$pname}".$gap."\n";
    }
    
    print OUT ";
END;


BEGIN CODONS;
	CODONPOSSET * CodonPositions = 
		1:1-".($maxSeqLength-2)."\\3,\n";
    print OUT "		2:2-".($maxSeqLength-1)."\\3,\n";
    print OUT "		3:3-".$maxSeqLength."\\3 ".$maxSeqLength.";\n";
    
    print OUT "	CODESET  * UNTITLED = Universal: all ;
END;


BEGIN ASSUMPTIONS;
	OPTIONS  DEFTYPE=unord PolyTcount=MINSTEPS ;
END;


BEGIN MacClade;
	Version 4.0  87;
	LastModified -921069252;
	FileSettings editor  '0' '0' '1' '1';
	Singles 100;
	Editor 00011001111111100100010010 '4' '12' Geneva '9' '294' '1' all;
	EditorPosition  '321' '487' '947' '1781';
	TreeWindowPosition  '46' '6' '1371' '2510';
	ListWindow Characters closed Geneva '9' '50' '10' '129' '406' 000;
	ListWindow Taxa closed Geneva '9' '50' '10' '129' '490' 100000000;
	ListWindow Trees closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow TypeSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow WtSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow ExSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow CharSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow TaxSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow CharPartitions closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow CharPartNames closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow WtSets closed Geneva '9' '50' '10' '276' '490' ;
	ChartWindowPosition  '52' '30' '1358' '2500';
	StateNamesSymbols closed Geneva '9' '10' '50' '30' '148' '220';
	WindowOrder  Data;
	OtherSymbols &/ 00 ?-;
	Correlation  '0' '0' '1000' '0' '0' 10011010;
	Salmo 00000001;
	EditorFile  '2';
	ExportHTML _ MOSS  '100' 110000;
	PrettyPrint 10;
	EditorToolsPosition  '1249' '46' '115' '165';
	TreeWindowProgram 10;
	TreeWindow 0000;
	Continuous  '0' '3' 1;
	Calculations 0000001;
	SummaryMode  '0' '0' 0;
	Charts  Geneva '9' (normal) 0010;
	NexusOptions  '0' '0' '50' 001011001;
	TipLabel  '1';
	TreeFont  Geneva '9' (normal);
	TreeShape  1.0 1.0 1110;
	TraceLabels 0101;
	ChartColors  '0' '0' '65535' '9' '0' 1;
	ChartBiggestSpot 1;
	ChartPercent 10;
	ChartBarWidth  '10' 1;
	ChartVerticalAxis 10101;
	ChartMinMax  '0';
	TraceAllChangesDisplay  '1' 1;
	BarsOnBranchesDisplay  '0' '0' '60000' '10000' '10000' '10000' '10000' '60000' '65000' '65000' '65000' '6' '1' 0000101;
	ContinuousBranchLabels 0;
	AllStatesBranchLabels 1;
	IndexNotation  '2' 1;
	PrintTree  10.00 '2' '2' '2' '2' '2' '2' '2' '2' '2' '2' '2' Geneva '9' (normal) Geneva '10' (normal) Geneva '9' (normal) Geneva '9' (normal) Geneva '9' (bold ) Geneva '9' (normal) Geneva '9' (normal) '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '1' '1' '1' '1' '0' '13' '-39' '4' '-40' '0' '1' '2' '1' '8' '0' '0' '0' '2' 1000111000000000000100000111000;
	MatchChar 00 .;
	EntryInterpretation 01;
	ColorOptions 00;
	TreeTools  '0' '5' '4' '0' '10' '4' '0' 00100111111101110;
	EditorTools  '0' '0' '0' '1000' '0' '0' '6' '3' '1' 100010101110011;
	PairAlign  '2' '2' '3' '2' '1' '1' '2' '1' '3' 1010;
	BothTools  '1';
END;


";
    
    print OUT "\n";
    
    close OUT;
    
    # outfile の改行コードを Unix から　Mac する．
    system ("perl -i -pe 's/\n/\r/g' $outfile");
    
    print "infile: $infile\n";
    print "outfile: $outfile\n";
}